#ifndef EXPR_PARSER_H
#define EXPR_PARSER_H 1
#include <string>
#include <iostream>
#include <deque>
#include <vector>
#include <map>

// Recursive parser of ariphmetic expressions,
// including variable x, y and standerd mathematical functions

// Scanner

class Token {
public:
    enum {
        NUMBER,
        PLUS,
        MINUS,
        MUL,
        DIV,
        POW,
        LPAR,
        RPAR,
        X,
        Y,
        NAME,
        COMMA,
        UMINUS,         // Unary minus == change sign operation
        ENDLINE,
        ILLEGAL
    };
    
    int lex;
    double number;
    std::string name;
    
    Token(
        int lexeme = ILLEGAL,
        double num = 0.,
        const std::string str = std::string()
    ):
        lex(lexeme),
        number(num),
        name(str)
    {}
};

std::ostream& operator<<(std::ostream& s, const Token& token);
    
class Scanner {
public:
    std::string inputLine;
    int curpos;
    int len;
    
    bool lexemeReady;
    Token lexeme;
    
    void init(const char* input);
    Token getToken();
    const Token& peekToken();
    
    Scanner():
       inputLine(),
       curpos(0),
       len(0),
       lexemeReady(false),
       lexeme()
    {}
       
private:
    void readToken();
};

class ParserException {
public:
    std::string txt;
    
    ParserException(const char* reason = ""):
        txt(reason)
    {}
    
    const char* what() const {
        return txt.c_str();
    }
};

// Reverse Polish Notation
class Rpn: public std::vector<Token> {
private:
    mutable std::deque<double> stack;
public:
    double evaluate(double x = 0., double y = 0.) const;
};

/* Context-free grammar

S -> F ENDLINE
F -> T E
E -> eps | + T E | - T E
T -> P G
G -> eps | * P G | / P G
P -> M K | - M K
K -> eps | POW M K
M -> number | x | y | ( F ) | NAME ( Args )
Args -> F H
H -> eps | , F

*/

class Parser {
public:
    Scanner scanner;
    Rpn rpn;
    
    Parser():
        scanner(),
        rpn()
    {}
    
    void init(const char* input);
    
    bool parse(std::string& message);
    
    double evaluate(double x = 0., double y = 0.) const;
private:
    void processS();
    void processF();
    void processE();
    void processT();
    void processG();
    void processP();
    void processK();
    void processM();
    void processArgs();
    void processH();
    
    void skipToken(int token);
};

#endif
