#include "drawarea.h"
#include "mainwindow.h"
#include <QPainter>
#include <QPainterPath>
#include <cmath>

double f(double x) {
    // return x*sin(x);
    return mainWindow->parser.evaluate(x);
}

DrawArea::DrawArea(QWidget *parent) :
    QWidget(parent),
    xmin(-8.),
    xmax(8.),
    ymin(-6.),
    ymax(6.)
{
    drawArea = this;
}

void DrawArea::paintEvent(QPaintEvent* /* event */) {
    QPainter painter(this);
    painter.fillRect(
        QRect(0, 0, width(), height()),
        QBrush(Qt::lightGray)
    );
    drawCoordinates(&painter);
    if (mainWindow->parseSuccess) {
        plotFunction(&painter, &f);
    }
}

void DrawArea::drawCoordinates(QPainter* painter) {
    QPen grayPen(Qt::gray, 1);
    QPen blackPen(Qt::black, 2);

    painter->setPen(grayPen);
    // painter->drawLine(QPointF(0., 0.), QPointF(300., 300.));

    double x = round(xmin);
    while (x <= round(xmax)) {
        QPointF p0 = mapToPixels(R2Point(x, ymin));
        QPointF p1 = mapToPixels(R2Point(x, ymax));
        painter->drawLine(p0, p1);
        x += 1.;
    }

    double y = round(ymin);
    while (y <= round(ymax)) {
        QPointF p0 = mapToPixels(R2Point(xmin, y));
        QPointF p1 = mapToPixels(R2Point(xmax, y));
        painter->drawLine(p0, p1);
        y += 1.;
    }

    painter->setPen(blackPen);
    painter->drawLine(
        mapToPixels(R2Point(xmin, 0.)),
        mapToPixels(R2Point(xmax, 0.))
    );
    painter->drawLine(
        mapToPixels(R2Point(0., ymin)),
        mapToPixels(R2Point(0, ymax))
    );
}

void DrawArea::plotFunction(
    QPainter* painter, double (*f)(double)
) {
    painter->setRenderHints(QPainter::Antialiasing);
    QPainterPath path;

    double sizeY = ymax - ymin;
    double maxValue = (ymin + ymax)/2. + sizeY*3.;
    double minValue = (ymin + ymax)/2. - sizeY*3.;

    double x = xmin;
    double dx = 0.001;
    double y = 0.;
    bool funcDefined = true;
    try {
        y = (*f)(x);
    } catch (const ParserException& /* e */) {
        funcDefined = false;
    }
    funcDefined = (
        funcDefined &&
        minValue <= y && y <= maxValue
    );
    R2Point p;
    if (funcDefined) {
        p = R2Point(x, y);
        path.moveTo(mapToPixels(p));
        x += dx;
    }
    while (x <= xmax) {
        bool funcDef = true;
        try {
            y = (*f)(x);
        } catch (const ParserException& /* e */) {
            funcDef = false;
        }
        funcDef = (
            funcDef &&
            minValue <= y && y <= maxValue
        );
        if (!funcDefined && funcDef) {
            p = R2Point(x, y);
            path.moveTo(mapToPixels(p));
        } else if (funcDefined && funcDef) {
            p = R2Point(x, y);
            path.lineTo(mapToPixels(p));
        }
        funcDefined = funcDef;
        x += dx;
    }
    painter->setPen(QPen(Qt::blue, 3));
    painter->drawPath(path);
}

void DrawArea::setCoordinates(double xMin, double xMax) {
    if (width() == 0 || height() == 0)
        return;
    if (xMax <= xMin)
        return;
    xmin = xMin;
    xmax = xMax;
    double aspect = double(width())/double(height());
    double ySize = (xmax - xmin)/aspect;
    ymin = (-ySize/2.);
    ymax = ySize/2.;
    update();
}

void DrawArea::resizeEvent(QResizeEvent* /* event */) {
    if (width() == 0 || height() == 0)
        return;
    setCoordinates(xmin, xmax);
}
