#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <string>

// Global objects
DrawArea* drawArea = nullptr;
MainWindow* mainWindow = nullptr;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    xMin(-8.),
    xMax(8.),
    parseSuccess(false),
    ui(new Ui::MainWindow)
{
    mainWindow = this;
    ui->setupUi(this);
    ui->xMin->setText(QString::number(xMin));
    ui->xMax->setText(QString::number(xMax));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_xMin_editingFinished()
{
    QString strXMin = ui->xMin->text();
    bool ok = false;
    double x_min = strXMin.toDouble(&ok);
    if (!ok)
        return;
    xMin = x_min;
}

void MainWindow::on_xMax_editingFinished()
{
    QString strXMax = ui->xMax->text();
    bool ok = false;
    double x_max = strXMax.toDouble(&ok);
    if (!ok)
        return;
    xMax = x_max;
}

void MainWindow::on_redrawButton_clicked()
{
    if (xMin < xMax)
        drawArea->setCoordinates(xMin, xMax);
    if (ui->expression->text() != "")
        on_parseButton_clicked();
    drawArea->update();
}

void MainWindow::on_parseButton_clicked()
{
    QString expr = ui->expression->text();
    QByteArray charExpr = expr.toUtf8();
    expression = charExpr.constData();

    parser.init(expression.c_str());
    std::string message;
    if (parser.parse(message)) {
        parseSuccess = true;
        ui->labelMessage->setText("Parsing: Success");
    } else {
        parseSuccess = false;
        ui->labelMessage->setText(message.c_str());
    }
    drawArea->update();
}

void MainWindow::on_expression_returnPressed()
{
    on_parseButton_clicked();
}

void MainWindow::on_xMax_returnPressed()
{
    on_xMax_editingFinished();
    if (xMin < xMax)
        on_redrawButton_clicked();
}

void MainWindow::on_xMin_returnPressed()
{
    on_xMin_editingFinished();
    if (xMin < xMax)
        on_redrawButton_clicked();
}
